use Module::Build;

Module::Build -> new
(
 module_name	=> 'Data::Page::Viewport',
 license		=> 'artistic',
 dist_abstract  => 'Scroll thru data a page, or just an item, at a time',
 dist_author	=> 'Ron Savage <ron@savage.net.au>',
 build_requires	=>
 {
	 Test::More => 0,
	 Test::Pod  => 0,
 },
 configure_requires =>
 {
	 Module::Build => 0.3800,
 },
 requires	=>
 {
	 Set::Window => 0,
 },
) -> create_build_script();
